@@ -35,7 +35,7 @@ pub struct OldRecordInfo {
 impl OldRecordInfo {
     fn new(recref: &RecRef, segment: SegmentId, record_page: u64, version: u16) -> OldRecordInfo {
         OldRecordInfo {
-            recref: recref.clone(),
+            recref: *recref,
             segment,
             record_page,
             version,
@@ -140,7 +140,7 @@ impl Address {
             return Err(PrepareError::from(x));
         }
 
-        let to_lock: Vec<_> = records.iter().map(|(_, id, _)| id.clone()).collect();
+        let to_lock: Vec<_> = records.iter().map(|(_, id, _)| *id).collect();
         if let Err(x) = self.record_locks.lock_all(&to_lock, timeout) {
             if let Err(e) = self.segment_locks.unlock_all_write(deleted) {
                 dbg!("unlock error: {}", e);
@@ -176,7 +176,7 @@ impl Address {
     }
     pub fn acquire_record_lock(&self, id: &RecRef) -> Result<(), TimeoutError> {
         let timeout = *self.config.transaction_lock_timeout();
-        self.record_locks.lock_all(&[id.clone()], timeout)?;
+        self.record_locks.lock_all(&[*id], timeout)?;
         Ok(())
     }
 
@@ -185,7 +185,7 @@ impl Address {
         Ok(())
     }
     pub fn release_record_lock(&self, id: &RecRef) -> PERes<()> {
-        self.record_locks.unlock_all(&[id.clone()])?;
+        self.record_locks.unlock_all(&[*id])?;
         Ok(())
     }
 
@@ -195,12 +195,6 @@ impl Address {
         Ok(())
     }
 
-    pub fn recompute_last_pages(&self) -> PERes<()> {
-        let mut segments = self.segments.write()?;
-        segments.recompute_last_pages(&self.allocator)?;
-        Ok(())
-    }
-
     pub fn check_persistent_records(
         &self,
         records: &[(SegmentId, RecRef, u16)],
@@ -215,7 +209,7 @@ impl Address {
                     return Err(PrepareError::VersionNotLastest);
                 }
             } else {
-                return Err(PrepareError::RecordNotFound(PersyId(recref.clone())));
+                return Err(PrepareError::RecordNotFound(PersyId(*recref)));
             }
         }
         Ok(current_record_pages)
@@ -389,11 +383,6 @@ impl Address {
         Ok(())
     }
 
-    pub fn flush_segments(&self) -> PERes<()> {
-        let mut segments = self.segments.write()?;
-        segments.flush_segments(&self.allocator)
-    }
-
     pub fn exists_segment(&self, segment: &str) -> PERes<bool> {
         Ok(self.segments.read()?.has_segment(segment))
     }
@@ -418,7 +407,7 @@ impl Address {
     pub fn read(&self, recref: &RecRef, segment: SegmentId) -> Result<Option<(u64, u16)>, ReadError> {
         if let Some(mut page) = self.allocator.load_page_not_free(recref.page)? {
             if recref.pos > ADDRESS_PAGE_SIZE - ADDRESS_ENTRY_SIZE {
-                return Err(ReadError::InvalidPersyId(recref.clone()));
+                return Err(ReadError::InvalidPersyId(*recref));
             }
             Ok(page.segment_read_entry(segment, recref.pos))
         } else {
