@@ -185,12 +185,9 @@ impl Journal {
     }
 
     pub fn start(&self) -> PERes<JournalId> {
-        let buffer = Self::write_buffer(&Start::new(), &JournalId::new(0, 0))?;
-        let mut jr = self.journal.lock()?;
-        self.required_space(buffer.len() as u32, &mut jr)?;
-        let val = Self::append_buffer(&mut jr, &buffer)?;
+        let val = self.internal_log(&Start::new(), &JournalId::new(0, 0), false)?;
         let id = JournalId::new(val.0, val.1);
-        jr.starts.push(&id);
+        self.journal.lock()?.starts.push(&id);
         Ok(id)
     }
 
@@ -302,33 +299,28 @@ impl Journal {
         Ok(())
     }
 
-    fn write_buffer(entry: &dyn JournalEntry, id: &JournalId) -> PERes<Vec<u8>> {
+    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<(u64, u32)> {
         let mut buffer = Vec::<u8>::new();
         buffer.write_u8(entry.get_type());
         buffer.write_varint_u64(id.page);
         buffer.write_varint_u32(id.pos);
         entry.write(&mut buffer)?;
-        Ok(buffer)
-    }
 
-    fn append_buffer(jr: &mut MutexGuard<JournalShared>, buffer: &[u8]) -> PERes<(u64, u32)> {
-        let cur_page = jr.last_page;
-        let cur_pos = jr.last_pos;
-        jr.current.seek(cur_pos);
-        jr.current.write_all(buffer);
-        jr.last_pos += buffer.len() as u32;
-        Ok((cur_page, cur_pos))
-    }
-
-    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<()> {
-        let buffer = Self::write_buffer(entry, id)?;
-        let mut jr = self.journal.lock()?;
-        self.required_space(buffer.len() as u32, &mut jr)?;
-        Self::append_buffer(&mut jr, &buffer)?;
-        if flush {
-            self.allocator.flush_page(jr.current.clone())?;
+        let cur_page;
+        let cur_pos;
+        {
+            let mut jr = self.journal.lock()?;
+            self.required_space(buffer.len() as u32, &mut jr)?;
+            cur_page = jr.last_page;
+            cur_pos = jr.last_pos;
+            jr.current.seek(cur_pos);
+            jr.current.write_all(&*buffer);
+            if flush {
+                self.allocator.flush_page(jr.current.clone())?;
+            }
+            jr.last_pos += buffer.len() as u32;
         }
-        Ok(())
+        Ok((cur_page, cur_pos))
     }
 
     pub(crate) fn recover<T>(&self, mut found: T) -> PERes<()>
@@ -763,11 +755,9 @@ mod tests {
         config::{Config, TxStrategy},
         discref::DiscRef,
         id::{RecRef, SegmentId},
+        io::ArcSliceRead,
     };
-    use std::{
-        io::{Cursor, Seek, SeekFrom},
-        sync::Arc,
-    };
+    use std::sync::Arc;
     use tempfile::Builder;
 
     #[test]
@@ -804,9 +794,10 @@ mod tests {
         to_write.write(&mut buffer).unwrap();
 
         let mut to_read = InsertRecord::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.segment, seg_id);
         assert_eq!(to_read.recref.page, 20);
         assert_eq!(to_read.recref.pos, 10);
@@ -822,9 +813,10 @@ mod tests {
         to_write.write(&mut buffer).unwrap();
 
         let mut to_read = ReadRecord::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.segment, seg_id);
         assert_eq!(to_read.recref.page, 20);
         assert_eq!(to_read.recref.pos, 10);
@@ -840,9 +832,10 @@ mod tests {
         to_write.write(&mut buffer).unwrap();
 
         let mut to_read = UpdateRecord::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.segment, seg_id);
         assert_eq!(to_read.recref.page, 20);
         assert_eq!(to_read.recref.pos, 10);
@@ -859,9 +852,10 @@ mod tests {
         to_write.write(&mut buffer).unwrap();
 
         let mut to_read = DeleteRecord::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.segment, seg_id);
         assert_eq!(to_read.recref.page, 20);
         assert_eq!(to_read.recref.pos, 10);
@@ -875,9 +869,10 @@ mod tests {
         let to_write = CreateSegment::new("some", seg_id, 20);
         to_write.write(&mut buffer).unwrap();
         let mut to_read = CreateSegment::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.name, "some");
         assert_eq!(to_read.segment_id, seg_id);
         assert_eq!(to_read.first_page, 20);
@@ -890,9 +885,10 @@ mod tests {
         let to_write = DropSegment::new("some", seg_id);
         to_write.write(&mut buffer).unwrap();
         let mut to_read = DropSegment::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.name, "some");
         assert_eq!(to_read.segment_id, seg_id);
     }
@@ -904,9 +900,10 @@ mod tests {
         let to_write = Metadata::new(&TxStrategy::VersionOnWrite, meta_id.clone());
         to_write.write(&mut buffer).unwrap();
         let mut to_read = Metadata::new(&TxStrategy::LastWin, Vec::new());
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.strategy, TxStrategy::VersionOnWrite);
         assert_eq!(to_read.meta_id, meta_id);
     }
@@ -918,9 +915,10 @@ mod tests {
         let to_write = NewSegmentPage::new(seg_id, 20, 30);
         to_write.write(&mut buffer).unwrap();
         let mut to_read = NewSegmentPage::default();
-        let mut cursor = Cursor::<&Vec<u8>>::new(&buffer);
-        to_read.read(&mut cursor).unwrap();
-        assert_eq!(cursor.seek(SeekFrom::Current(0)).unwrap(), (buffer.len()) as u64);
+        let len = buffer.len();
+        let mut reader = ArcSliceRead::new_vec(buffer);
+        to_read.read(&mut reader).unwrap();
+        assert_eq!(reader.cursor(), len);
         assert_eq!(to_read.segment, seg_id);
         assert_eq!(to_read.page, 20);
         assert_eq!(to_read.previous, 30);
