@@ -34,13 +34,13 @@ pub enum EntryCase {
 impl SnapshotEntry {
     pub fn change(id: &RecRef, pos: u64, version: u16) -> SnapshotEntry {
         SnapshotEntry {
-            id: id.clone(),
+            id: *id,
             case: EntryCase::Change(Change { pos, version }),
         }
     }
     pub fn insert(id: &RecRef) -> SnapshotEntry {
         SnapshotEntry {
-            id: id.clone(),
+            id: *id,
             case: EntryCase::Insert,
         }
     }
@@ -169,20 +169,11 @@ impl InternalSnapshots {
                 }
                 if let Some(entries) = tx.entries {
                     for record in entries {
-                        if let Entry::Occupied(mut v) = self.mapping.entry(record.id) {
-                            match v
-                                .get()
-                                .binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
+                        if let Some(v) = self.mapping.get_mut(&record.id) {
+                            if let Ok(index) =
+                                v.binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
                             {
-                                Ok(index) => {
-                                    v.get_mut().truncate(index);
-                                    if v.get().is_empty() {
-                                        v.remove();
-                                    }
-                                }
-                                Err(_index) => {
-                                    v.remove();
-                                }
+                                v.truncate(index);
                             }
                         }
                     }
@@ -308,7 +299,7 @@ impl Snapshots {
             };
 
             let snapshot_sequence = lock.snapshot_sequence;
-            match lock.mapping.entry(entry.id.clone()) {
+            match lock.mapping.entry(entry.id) {
                 Entry::Occupied(mut v) => {
                     let vec = v.get_mut();
                     if let Err(index) = vec.binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
@@ -587,53 +578,6 @@ mod tests {
         assert_eq!(snap.read(last, &RecRef::new(10, 2)).unwrap(), None);
     }
 
-    #[test]
-    fn test_clanup_after_release() {
-        let snap = Snapshots::new();
-        let mut snapshots_id = Vec::new();
-        for s in 0..100 {
-            let mut records = Vec::new();
-            let mut freed_pages = Vec::new();
-            for x in 0..3 {
-                records.push(SnapshotEntry::change(&RecRef::new(s % 10, x), x as u64, x as u16));
-                freed_pages.push(FreedPage::new(x as u64));
-            }
-            let tx = snap
-                .snapshot(records.clone(), freed_pages, JournalId::new(0, 0))
-                .unwrap();
-            snapshots_id.push(tx);
-        }
-        for s in snapshots_id.into_iter().rev() {
-            snap.release(s).unwrap();
-        }
-        {
-            let data = snap.lock.lock().unwrap();
-            assert_eq!(data.active_snapshots.len(), 0);
-            assert_eq!(data.mapping.len(), 0);
-        }
-        // Same code twice this time release the snapshot in direct order
-        let mut snapshots_id = Vec::new();
-        for s in 0..100 {
-            let mut records = Vec::new();
-            let mut freed_pages = Vec::new();
-            for x in 0..3 {
-                records.push(SnapshotEntry::change(&RecRef::new(s % 10, x), x as u64, x as u16));
-                freed_pages.push(FreedPage::new(x as u64));
-            }
-            let tx = snap
-                .snapshot(records.clone(), freed_pages, JournalId::new(0, 0))
-                .unwrap();
-            snapshots_id.push(tx);
-        }
-        for s in snapshots_id {
-            snap.release(s).unwrap();
-        }
-
-        let data = snap.lock.lock().unwrap();
-        assert_eq!(data.active_snapshots.len(), 0);
-        assert_eq!(data.mapping.len(), 0);
-    }
-
     #[test]
     fn test_snapshot_release_clear() {
         let snap = Snapshots::new();
@@ -652,8 +596,6 @@ mod tests {
         snap.release(snaps[2]).expect("release works");
         let solved = snap.solve_segment_id(snaps[3], "one").expect("solve id correctly");
         assert_eq!(solved, Some(SegmentId::new(3)));
-        let data = snap.lock.lock().unwrap();
-        assert_eq!(data.active_snapshots.len(), 7);
     }
 
     #[test]
