@@ -8,6 +8,7 @@ use crate::{
         config::{IndexType, ValueMode, INDEX_DATA_PREFIX, INDEX_META_PREFIX},
         iter::TxIndexIter,
         keeper::TxIndexRawIter,
+        tree::nodes::Value,
         value_iter::ValueIter,
     },
     persy::{IndexInfo, PersyImpl, TxFinalize},
@@ -112,7 +113,8 @@ impl Transaction {
         Ok(self.persy_impl.solve_segment_id_tx(self.tx(), segment)?)
     }
 
-    /// Resolves the index name to a IndexId, considering the transaction
+    /// Resolves the index name to a IndexId, considering the transaction,
+    /// this has no public use as today, but may be used in future.
     ///
     /// # Example
     ///
@@ -123,7 +125,7 @@ impl Transaction {
     /// # let mut tx = persy.begin()?;
     /// let mut tx = persy.begin()?;
     /// tx.create_index::<u8,u8>("my_new_index", ValueMode::Cluster)?;
-    /// let segment_id = tx.solve_index_id("my_new_index")?;
+    /// let index_id = tx.solve_index_id("my_new_index")?;
     /// # tx.prepare()?.commit()?;
     /// # Ok(())
     /// # }
@@ -362,7 +364,9 @@ impl Transaction {
         let index_id = self
             .solve_index_id(index_name)
             .map_err(|e| PE::PE(IndexOpsError::from(e.error())))?;
-        Ok(self.persy_impl.put::<K, V>(tx_mut(&mut self.tx), index_id, k, v)?)
+        Ok(self
+            .persy_impl
+            .put::<K::Wrapper, V::Wrapper>(tx_mut(&mut self.tx), index_id, k.wrap(), v.wrap())?)
     }
 
     /// Remove a key and optionally a specific value from an index following the value mode strategy.
@@ -389,7 +393,12 @@ impl Transaction {
         let index_id = self
             .solve_index_id(index_name)
             .map_err(|e| PE::PE(IndexOpsError::from(e.error())))?;
-        Ok(self.persy_impl.remove::<K, V>(tx_mut(&mut self.tx), index_id, k, v)?)
+        Ok(self.persy_impl.remove::<K::Wrapper, V::Wrapper>(
+            tx_mut(&mut self.tx),
+            index_id,
+            k.wrap(),
+            v.map(|rv| rv.wrap()),
+        )?)
     }
 
     /// Get a value or a group of values from a key considering changes in transaction.
@@ -419,7 +428,9 @@ impl Transaction {
         let index_id = self
             .solve_index_id(index_name)
             .map_err(|e| IndexChangeError::from(e.error()))?;
-        let entry = self.persy_impl.get_tx::<K, V>(tx_mut(&mut self.tx), index_id, k)?;
+        let entry: Option<Value<V::Wrapper>> =
+            self.persy_impl
+                .get_tx::<K::Wrapper, V::Wrapper>(tx_mut(&mut self.tx), index_id, &k.clone().wrap())?;
         Ok(ValueIter::from(entry))
     }
 
@@ -488,9 +499,10 @@ impl Transaction {
         let imp = self.persy_impl.clone();
 
         let tx = self.tx_mut();
-        let (vm, raw) = imp.range_tx(tx, index_id.clone(), (range.start_bound(), range.end_bound()))?;
-        let tx_iter = tx.index_range::<K, V, R>(index_id.clone(), range);
-        let tx_raw = TxIndexRawIter::new(index_id, tx_iter, Some(raw), vm);
+        let rr = PersyImpl::map_index_range_bounds(range);
+        let (vm, raw) = imp.range_tx(tx, index_id.clone(), rr.clone())?;
+        let tx_iter = tx.index_range::<K::Wrapper, V::Wrapper, _>(index_id.clone(), rr);
+        let tx_raw = TxIndexRawIter::new(index_id, index_name.to_owned(), tx_iter, Some(raw), vm);
         Ok(TxIndexIter::new(tx_raw, self))
     }
 
